#include <vfw.h>

#include "stdafx.h"
#include "capture.h"
#include "VideoProtocol.h"
#include "convert.h"
#include "display.h"


Capture *cap;
FILE *clog;

int Capture::init(HWND parent, int drv, BOOL test)
{
	CAPTUREPARMS params;
	
#ifdef	_DEBUG
	clog = fopen("capture.log", "w");
#endif

	cap = this;
	buffer = NULL;
	state = STOP;
	display = displayOpen(MAX_USERS);
	if(test == TRUE)
		displayHide(MAX_USERS);

	// capture window
	capWnd = capCreateCaptureWindow(
		(LPSTR) "Mirror",
		WS_CHILD /*| WS_VISIBLE*/,
		0, 0, 176, 144,
		display->m_hWnd,
		23);

	if(capWnd == NULL)
		return 0;

	// connect to capture driver
	if(!capDriverConnect(capWnd, drv))
	{
		DestroyWindow(capWnd);
		displayClose(MAX_USERS);
		return 0;
	}

	// set video format
	if(!checkFormat())
	{
		if(!test)
		{
			DestroyWindow(capWnd);
			displayClose(MAX_USERS);
		}
		return 0;
	}

	// set callback for frame
	if(!capSetCallbackOnVideoStream(capWnd, frameClb))
		return 0;

	// set setup
	capCaptureGetSetup(capWnd, &params, sizeof(CAPTUREPARMS));
    params.dwRequestMicroSecPerFrame = 1000 * 1000 / FPS; 
    params.fMakeUserHitOKToCapture = FALSE; 
    params.wPercentDropForError = 1; 
    params.fYield = TRUE;
    params.wNumVideoRequested = 20; 
    params.fCaptureAudio = FALSE; 
    params.wNumAudioRequested = 1; 
    params.fLimitEnabled = FALSE; 
    params.wTimeLimit = 30; 
	params.fAbortLeftMouse = FALSE; 
    params.fAbortRightMouse = FALSE; 

	if(!capCaptureSetSetup(capWnd, &params, sizeof(CAPTUREPARMS)))
		return 0;

	capPreview(capWnd, FALSE);

	hCapEvent = CreateEvent(NULL, TRUE, TRUE, "capture event");

	size = info->bmiHeader.biWidth * info->bmiHeader.biHeight * info->bmiHeader.biBitCount / 8;
	buffer = new char[size];
	
	return 1;
}

void Capture::close()
{
	capSetCallbackOnFrame(capWnd, NULL);
	capDriverDisconnect(capWnd);
	DestroyWindow(capWnd);
	displayClose(MAX_USERS);
	GlobalFree(info);
	if(buffer != NULL)
		delete [] buffer;
	
#ifdef	_DEBUG
	fclose(clog);
#endif
}

int Capture::checkFormat(void)
{
	DWORD info_size;
	char text[100];

	// getting current format settings
	info_size = capGetVideoFormatSize(capWnd);
	info = (BITMAPINFO *)GlobalAlloc(GPTR, info_size);
	capGetVideoFormat(capWnd, info, info_size);

	switch(info->bmiHeader.biCompression)
	{
	case BI_RGB:
		if(info->bmiHeader.biBitCount != 24)
		{
			MessageBox(NULL, "Unsupported video format(wrong bits/pixel value).\nPlease, change your video format to RGB24 or YUV9.", "Error", MB_OK | MB_ICONEXCLAMATION);
			return 0;
		}
		break;

	case BI_YUV9:
		break;

	default:
		MessageBox(NULL, "Unsupported video format.\nPlease, change your video format to RGB24 or YUV9.", "Error", MB_OK | MB_ICONEXCLAMATION);
		return 0;
		break;
	}

	if((info->bmiHeader.biWidth < 176) && (info->bmiHeader.biHeight < 144))
	{
		info->bmiHeader.biWidth = 176;
		info->bmiHeader.biHeight = 144;
		capSetVideoFormat(capWnd, info, info_size);
		capGetVideoFormat(capWnd, info, info_size);
		if((info->bmiHeader.biWidth < 176) || (info->bmiHeader.biHeight < 144))
		{
			wsprintf(text, "Unsupported video format (wrong picture size %d x %d).\nPlease, change picture size to at least 176 x 144", info->bmiHeader.biWidth, info->bmiHeader.biHeight);
			MessageBox(NULL, text, "Error", MB_OK | MB_ICONEXCLAMATION);
			return 0;
		}
	}


	return 1;
}

int Capture::start(void)
{
	if(state == STOP)
	{
		state = START;
		if(!capCaptureSequenceNoFile(capWnd))
		{
			MessageBox(NULL, "Capture start error", "Error", MB_OK);
			return 0;
		}
		displayShow(MAX_USERS);
	}

	return 1;
}

void Capture::stop()
{
	if(state == START)
	{
		capCaptureStop(capWnd);
		displayHide(MAX_USERS);
		state = STOP;
	}
}

LRESULT PASCAL Capture::frameClb(HWND hWnd, LPVIDEOHDR lpVHdr)
{
	WaitForSingleObject(cap->hCapEvent, INFINITE);
	ResetEvent(cap->hCapEvent);
	
/*
#ifdef	_DEBUG
	int tmp = GetTickCount();
	fprintf(clog, "Frame callback time: %d\n", tmp);
	fflush(clog);
#endif	
*/	
	CopyMemory(cap->buffer, lpVHdr->lpData, cap->size);

	SetEvent(cap->hCapEvent);
	return TRUE;
}
 
int Capture::getFrame(unsigned char **frame, int *type, int *dx, int *dy)
{
	WaitForSingleObject(hCapEvent, INFINITE);
	ResetEvent(hCapEvent);
	
	*dx = info->bmiHeader.biWidth;
	*dy = info->bmiHeader.biHeight;
	*type = info->bmiHeader.biCompression;
	*frame = new unsigned char[size];
	CopyMemory(*frame, buffer, size);

	SetEvent(hCapEvent);

	return 1;
}

void Capture::formatDlg()
{
	if(capWnd != NULL)
		capDlgVideoFormat(capWnd);
}


void Capture::sourceDlg()
{
	if(capWnd != NULL)
		capDlgVideoSource(capWnd);
}
