/*********************************************************************
 *
 * File:         daemon.c
 *
 * Description:  Daemon for conference engine
 *
 * Author:       Tomasz Stachowiak, NPAC
 *
 * Date:         10-14-96
 *
 *********************************************************************/
#include "stdafx.h"

#include <stdio.h>
#include <winsock.h>

#include "global.h"
#include "session_manage.h"
#include "BVtango.h"
#include "BVtangoDlg.h"
#include "Conference.h"
#include "daemon.h"

extern FILE *log;

Conference *conference;
char localHost[50];   /* local host name */

/* System sockets */
int serverSocket;          /* main listening socket */
int tmpSocket;             /* temporary socket */
int inviteSocket;             /* invitation receive and handle socket */
int mbrSocket[MAX_MEMBERS];  /* member sockets */

int dbState;

struct sockaddr_in tmpAddr;

struct sockaddr_in addr;   /* address to bind to */

void nic(...)
{
}


int readAll(int fd, void* buffer, size_t len)
{
  int st;
  
  do
    {
      st = recv( fd, (char *)buffer, len, 0 );
      if( st > 0 )
        {
	  len -= st;
	  buffer = ( char* )buffer + st;
        } 
      else
        {
	  if( st < 0 )
            {
	      perror( "readAll" );
	      return -1;
            }
	  return 0;
        }
    }
  while( len );

  return 1;
}

int readMsgHeader(int sock, MsgHeader *header)
{
  char buf[MSG_HEADER_LENGTH];
  int err;
  struct sockaddr_in tmp;
  int len;

  PRINTF(log, "Engine: reading header\n");
  err = readAll(sock, buf, MSG_HEADER_LENGTH);
  if(err == 0)
    {
      fprintf( stderr, "Engine: connection broken\n");
      return err;
    }
  if(err < 0)
    {
      fprintf( stderr, "Engine: reading error\n");
      return err;
    }

  header->tag = buf[0];
  header->session = buf[1];
  header->sender.handle = buf[2];
  strcpy(header->sender.user, &buf[3]);
  header->sender.host = *(int *)&buf[23];
  //header->sender.host = ntohl(*(int *)&buf[23]);
  header->receiver.handle = buf[27];
  strcpy(header->receiver.user, &buf[28]);
  header->receiver.host = *(int *)&buf[48];
  //header->receiver.host = ntohl(*(int *)&buf[48]);
  header->dataLength = ntohl(*(int *)&buf[52]);

  len = sizeof(struct sockaddr_in);
  getpeername(sock, (struct sockaddr *)&tmp, &len);
  header->sender.host = tmp.sin_addr.s_addr;


  return err;
}

int getFreeMbrSocket()
{
  int found=0;
  int i=1;

  while((i < MAX_MEMBERS) && (!found))
    {
      if((mbrSocket[i] < 0) && (i != getLocalHandle()))
	found = 1;
      else
	i++;
    }
  if(found)
    return i;
  else
    return -1;
}


void setFD(fd_set *set)
{
  int i;

  FD_ZERO(set);

  for(i=0; i<MAX_MEMBERS; i++)
    if(mbrSocket[i] >= 0)
      FD_SET(mbrSocket[i], set);

  if(inviteSocket >= 0)
    FD_SET(inviteSocket, set);
    
  if(tmpSocket >= 0)
    FD_SET(tmpSocket, set);

  FD_SET(serverSocket, set);
}


int openServerSocket()
{
  if ((serverSocket = socket(AF_INET, SOCK_STREAM, 0)) < 0)
    fprintf( stderr, "Engine: can't open daemon socket\n");

  addr.sin_family = AF_INET;
  addr.sin_addr.s_addr = htonl(INADDR_ANY);
  addr.sin_port        = htons(DAEMON_PORT);

  if(bind(serverSocket, (struct sockaddr *)&addr, sizeof(addr)) < 0)
    {
      fprintf( stderr, "Engine: can't bind daemon socket\n");
      quitDaemon();
      closesocket(serverSocket);
      exit(0);
    }

  listen(serverSocket, 5);

  return 1;
}

/* Daemon initialization */
void initDaemon(Conference *conf)
{
  int i;
  WSADATA wsaData;	
  
#ifdef _DEBUG
	log = fopen("BuenaVista.log", "w");
#endif

	conference = conf;

	PRINTF(log, "Daemon init\n");

	if(WSAStartup(257, &wsaData) != 0)
	  PRINTF(log, "Engine: can't startup winsock\n");

  for(i=0; i<MAX_MEMBERS; i++)
    mbrSocket[i] = -1;

  tmpSocket = -1;
  inviteSocket = -1;

  openServerSocket();

  //getHostFullName(localHost, 50);

  sessionAbandon();

  dbState = DbStateIdle;

	PRINTF(log, "Daemon initialized\n");
}


/* Daemon quitting */
void quitDaemon()
{
  int i;

  sessionAbandon();

  for(i=0; i<MAX_MEMBERS; i++)
    if(mbrSocket[i] >= 0)
      {
		closesocket(mbrSocket[i]);
		mbrSocket[i] = -1;
      }

  if(inviteSocket >= 0)
    {
      closesocket(inviteSocket);
      inviteSocket = -1;
    }

  if(tmpSocket >= 0)
    {
      closesocket(tmpSocket);
      tmpSocket = -1;
    }

#ifdef _DEBUG
	fclose(log);
#endif
}

/**************************************************************************
 *
 * Main socket reading procedure
 *
 **************************************************************************/
void runDaemon()
{
  fd_set set;
  struct timeval timeout;
  int len;
  int i;

  len = sizeof(struct sockaddr_in);
  timeout.tv_sec = 1;
  timeout.tv_usec = 0;
  while(1)
    {
      setFD(&set);
      if(select(FD_SETSIZE, &set, 0, 0, &timeout) > 0)
	{
	  for(i=0; i<MAX_MEMBERS; i++)
	      if((mbrSocket[i] >= 0) && (FD_ISSET(mbrSocket[i], &set)))
		readMemberMessage(i);
	  
	  if((tmpSocket >= 0) && (FD_ISSET(tmpSocket, &set)))
	    readNewMessage();

	  if(FD_ISSET(serverSocket, &set))
	    {
	      if(tmpSocket < 0)
		{
		  tmpSocket = accept(serverSocket, (struct sockaddr *)&tmpAddr, &len);	    
		  if(tmpSocket < 0)
		    fprintf( stderr, "Engine: accept error\n");
		  else
		    PRINTF(log, "Accepted\n");
		}
	    }
	}
    }
}

/*********************************************************************
 *
 * Incoming messages reading procedures
 *
 *********************************************************************/
void readMemberMessage(int no)
{
  char *msg;
  MsgHeader header;
  int err;

  PRINTF(log, "Engine: read member message\n");
  err = readMsgHeader(mbrSocket[no], &header);
  if(err == 0)
    {
      PRINTF(log, "Engine: member left %d\n", no);
      sessionDeleteMember(no);
      closesocket(mbrSocket[no]);
      mbrSocket[no] = -1;
      /* member left handle */
      return;
    }

  if(err < 0)
    {
      fprintf( stderr, "Engine: data reading error\n");
 	  sessionDeleteMember(no);
	  closesocket(mbrSocket[no]);
	  mbrSocket[no] = -1;
     return;
    }

  if(header.dataLength > 0)
    {
      msg = (char *)malloc(header.dataLength);
      err = readAll(mbrSocket[no], msg, header.dataLength);
      if(err == 0)
	{
	  fprintf( stderr, "Engine: member left\n");
	  sessionDeleteMember(no);
	  closesocket(mbrSocket[no]);
	  mbrSocket[no] = -1;
	  /* member left handle */
	  return;
	}
      
      if(err < 0)
	{
	  fprintf( stderr, "Engine: data reading error\n");
	  sessionDeleteMember(no);
	  closesocket(mbrSocket[no]);
	  mbrSocket[no] = -1;
	  return;
	}
    }
  else
    msg = NULL;
  
  handleMemberMessage(no, &header, msg, header.dataLength);
  
  free(msg);
}


void readNewMessage()
{
  MsgHeader header;
  char *msg;
  int err;

  PRINTF(log, "Engine: read new message\n");
  err = readMsgHeader(tmpSocket, &header);
  if(err == 0)
    {
      closesocket(tmpSocket);
      tmpSocket = -1;
      return;
    }
  if(err < 0)
    {
      fprintf( stderr, "Engine: data reading error\n");
      closesocket(tmpSocket);
      tmpSocket = -1;
      return;
    }

  PRINTF(log, "Reading message length %d\n", header.dataLength );
  if(header.dataLength > 0)
    {
      msg = (char *)malloc(header.dataLength);
      err = readAll(tmpSocket, msg, header.dataLength);
      if(err == 0)
	{
	  closesocket(tmpSocket);
	  tmpSocket = -1;
	  return;
	}
      if(err < 0)
	{
	  fprintf( stderr, "Engine: data reading error\n");
      closesocket(tmpSocket);
      tmpSocket = -1;
	  return;
	}
    }
  else
    msg = NULL;
  
  handleNewMessage(&tmpAddr, &header, msg, header.dataLength);
  
  free(msg);
  closesocket(tmpSocket);
  tmpSocket = -1;
}

void readInviteMessage()
{
  MsgHeader header;
  char *msg;
  int err;

  PRINTF(log, "Engine: read invite message\n");
  err = readMsgHeader(inviteSocket, &header);
  if(err == 0)
    {
      /* handle invitor exit */
    }
  if(err < 0)
    {
      fprintf( stderr, "Engine: data reading error\n");
      return;
    }

  PRINTF(log, "Engine: Reading message\n");
  if(header.dataLength > 0)
    {
      msg = (char *)malloc(header.dataLength);
      err = readAll(inviteSocket, msg, header.dataLength);
      if(err == 0)
	{
	  /* handle invitor exit */
    	}
      if(err < 0)
	{
	  fprintf( stderr, "Engine: data reading error\n");
	  return;
	}
    }
  else
    msg = NULL;
  
  if(header.tag == reqRemoteSessionAddRest)
    remoteClbSessionAddRest(1, &header, msg, header.dataLength);
  else
    fprintf( stderr, "Engine: Unknown remote tag\n");
  
  free(msg);
}



/*********************************************************************
 *
 * Incoming messages handling procedures
 *
 *********************************************************************/
void handleNewMessage(struct sockaddr_in *addr, MsgHeader *header, char *msg, int len)
{
  switch((NewTag)header->tag)
    {
    case reqNewRemoteSessionInvite: 
      PRINTF(log, "Engine: remote session invited\n");
      newClbSessionInvite(addr, header, msg, len);
      break;

    case reqNewRemoteSessionConnect: 
      PRINTF(log, "Engine: remote session connect\n");
      newClbSessionConnect(addr, header, msg, len);
      break;

    default:
      fprintf( stderr, "Engine: Unknown new tag\n");
    }
}

void handleMemberMessage(int no, MsgHeader *header, char *msg, int len)
{
  switch((MemberTag)header->tag)
    {
    case reqRemoteSessionAccept:
      PRINTF(log, "Engine: remote session accept\n");
      remoteClbSessionAccept(no, header, msg, len);
      break;

    case reqRemoteSessionReject:
      PRINTF(log, "Engine: remote: session reject\n");
      remoteClbSessionReject(no, header, msg, len);
      break;

    case reqRemoteSessionAddMember:
      PRINTF(log, "Engine: remote session add member\n");
      remoteClbSessionAddMember(no, header, msg, len);
      break;

    case reqRemoteSessionAddRest:
      PRINTF(log, "Engine: remote session add rest\n");
      remoteClbSessionAddRest(no, header, msg, len);
      break;

    case reqRemoteSessionQuit:
      PRINTF(log, "Engine: remote session quit\n");
      remoteClbSessionQuit(no, header, msg, len);
      break;

    case reqRemoteSessionSendAppData:
      PRINTF(log, "Engine: remote session send applications data\n");
      remoteClbSessionSendAppData(no, header, msg, len);
      break;

    case reqRemoteSessionSendToApp:
      PRINTF(log, "Engine: remote session send to application\n");
      remoteClbSessionSendToApp(no, header, msg, len);
      break;

    case reqRemoteSessionStartApp:
      PRINTF(log, "Engine: remote session start application\n");
      remoteClbSessionStartApp(no, header, msg, len);
      break;

    default:
      fprintf( stderr, "Engine: Unknown member tag %d\n", header->tag);
    }
}








































































