#include <dsound.h>

#include "stdafx.h"
#include "DSaudio.h"

#define DELAY 3200
#define AUDIO_CHUNK_SIZE 1600


int DSbuffer::init(LPDIRECTSOUND lpDirectSound, WAVEFORMATEX *pformat)
{
	HRESULT hr;
	
    state = STOP;

	memset(&dsbdesc, 0, sizeof(DSBUFFERDESC)); // Zero it out.
    dsbdesc.dwSize = sizeof(DSBUFFERDESC);
    // Need default controls (pan, volume, frequency).
    dsbdesc.dwFlags = DSBCAPS_CTRLDEFAULT | DSBCAPS_GLOBALFOCUS; 
    
	// 10-second buffer.
	size = 30 * pformat->nAvgBytesPerSec; 
	pos = DELAY;
	
	dsbdesc.dwBufferBytes = size; 
    dsbdesc.lpwfxFormat = (LPWAVEFORMATEX)pformat;

    // Create buffer.
    hr = lpDirectSound->CreateSoundBuffer(&dsbdesc, &DSbuf, NULL);
    if(DS_OK == hr) {
        // Succeeded. Valid interface is in *lplpDsb.
		DSbuf->SetVolume(0);

		start();

		return 1;
    } 
	else 
	{
        // Failed.
        DSbuf = NULL;
        return 0;
    }
}

int DSbuffer::start()
{
	HRESULT hr;

	if(state == STOP)
	{
		hr = DSbuf->Play(0, 0, DSBPLAY_LOOPING);
		if(hr != DS_OK)
			return 0;
		state = START;
	}
	
	return 1;
}

int DSbuffer::stop()
{
    LPVOID lpvPtr1;
    LPVOID lpvPtr2;
    DWORD dwBytes1; 
    DWORD dwBytes2;
	HRESULT hr;

	if(state == START)
	{
		hr = DSbuf->Stop();
		if(hr != DS_OK)
			return 0;

		DSbuf->Lock(0, 0, &lpvPtr1, &dwBytes1, &lpvPtr2, &dwBytes2, DSBLOCK_ENTIREBUFFER);
		ZeroMemory(lpvPtr1, dwBytes1);
		if(dwBytes2 > 0)
			ZeroMemory(lpvPtr2, dwBytes2);
		DSbuf->Unlock(lpvPtr1, dwBytes1, lpvPtr2, dwBytes2);

		state = STOP;
	}
	
	return 1;
}

int DSbuffer::play(void *buf)
{
    LPVOID lpvPtr1;
    LPVOID lpvPtr2;
    DWORD dwBytes1; 
    DWORD dwBytes2;
    HRESULT hr;
	DWORD playPos;
	DWORD writePos;
    
	if(state == STOP)
		start();

	// Obtain write pointer.
    DSbuf->GetCurrentPosition(&playPos, &writePos);
	if((pos - playPos) % size > 5*DELAY)
	{
		pos = (playPos + DELAY) % size;
	}

	hr = DSbuf->Lock(pos, 3*AUDIO_CHUNK_SIZE, &lpvPtr1, 
        &dwBytes1, &lpvPtr2, &dwBytes2, 0/*DSBLOCK_FROMWRITECURSOR*/);

    // If DSERR_BUFFERLOST is returned, restore and retry lock.
    if(DSERR_BUFFERLOST == hr) {

        DSbuf->Restore();
        hr = DSbuf->Lock(pos, 20*AUDIO_CHUNK_SIZE, &lpvPtr1, &dwBytes1, &lpvPtr2, &dwBytes2, 0/*DSBLOCK_FROMWRITECURSOR*/);
    }
    if(DS_OK == hr) 
	{
        // Write to pointers.
        if(dwBytes2 == 0)
		{
			CopyMemory(lpvPtr1, buf, AUDIO_CHUNK_SIZE);
			ZeroMemory((char *)lpvPtr1 + AUDIO_CHUNK_SIZE, 2*AUDIO_CHUNK_SIZE);
		}
		else
		{
			if(dwBytes1 < AUDIO_CHUNK_SIZE)
			{
				CopyMemory(lpvPtr1, buf, dwBytes1);
				CopyMemory((char *)lpvPtr2, (char *)buf+dwBytes1, AUDIO_CHUNK_SIZE-dwBytes1);
				ZeroMemory((char *)lpvPtr2+AUDIO_CHUNK_SIZE-dwBytes1, dwBytes2-AUDIO_CHUNK_SIZE+dwBytes1);
			}
			else
			{
				CopyMemory(lpvPtr1, buf, AUDIO_CHUNK_SIZE);
				ZeroMemory((char *)lpvPtr1+AUDIO_CHUNK_SIZE, dwBytes1-AUDIO_CHUNK_SIZE);
				ZeroMemory((char *)lpvPtr2, dwBytes2);
			}
		}
		
		pos = (pos + AUDIO_CHUNK_SIZE) % size;
    }
	else
	{
		pos = (pos + AUDIO_CHUNK_SIZE) % size;
		return 0;
	}

	// Release the data back to DirectSound.
    hr = DSbuf->Unlock(lpvPtr1, dwBytes1, lpvPtr2, dwBytes2);
    if(DS_OK == hr) {
        // Success.
		return 1;
	}
    // Lock, Unlock, or Restore failed.
	
	return 0;
}

void DSbuffer::close()
{
	DSbuf->Stop();
	DSbuf->Release();
}


void DSmultiAudio::setFormat()
{
    memset(&format, 0, sizeof(WAVEFORMATEX));
    format.wFormatTag = WAVE_FORMAT_PCM;
	format.nChannels = 1;
	format.nSamplesPerSec = 8000;
    format.nAvgBytesPerSec = 16000; 
    format.nBlockAlign = 2; 
	format.wBitsPerSample = 16;
}

int DSmultiAudio::createPrimaryBuf()
{
	HRESULT hr;
	
	setFormat();
	
	memset(&dsbdesc, 0, sizeof(DSBUFFERDESC)); // Zero it out.
    dsbdesc.dwSize = sizeof(DSBUFFERDESC);
    dsbdesc.dwFlags = DSBCAPS_PRIMARYBUFFER;
    // Buffer size is determined by sound hardware.
    dsbdesc.dwBufferBytes = 0; 
    dsbdesc.lpwfxFormat = NULL; // Must be NULL for primary buffers.

	hr = lpDirectSound->CreateSoundBuffer(&dsbdesc, &DSprimaryBuffer, NULL);
    if(DS_OK == hr) {
		// Succeeded. Set primary buffer to desired format.
		hr = DSprimaryBuffer->SetFormat(&format);
		if(DS_OK == hr) {
			// If you want to know the buffer size, call GetCaps.
			/*
			dsbcaps.dwSize = sizeof(DSBCAPS);

			(*lplpDsb)->lpVtbl->GetCaps(*lplpDsb, &dsbcaps);
			*lpdwBufferSize = dsbcaps.dwBufferBytes;
			*/
			return 1;
		}
    }

	return 0;
}

int DSmultiAudio::init(HWND hwnd)
{
	int i;
	
	if(DS_OK == DirectSoundCreate(NULL, &lpDirectSound,
		NULL)) {
		// Creation succeeded.
		lpDirectSound->SetCooperativeLevel(hwnd, DSSCL_EXCLUSIVE);

		if(!createPrimaryBuf())
			return 0;
		
		for(i=0; i<MAX_STREAMS; i++)
			buffer[i] = NULL;

		return 1;

	} else {
		// Creation failed.
		return 0;
	}	
	
	return 0;
}

void DSmultiAudio::close()
{
	lpDirectSound->Release();
}


int DSmultiAudio::play(int id, void *buf)
{
	if(buffer[id] == NULL)
	{
		buffer[id] = new DSbuffer();
		buffer[id]->init(lpDirectSound, &format);
	}

	return buffer[id]->play(buf);
}

void DSmultiAudio::closeBuf(int id)
{
	if(buffer[id] != NULL)
	{
		buffer[id]->close();
		delete buffer[id];
		buffer[id] = NULL;
	}
}

int DSmultiAudio::start(int no)
{

	if(buffer[no] != NULL)
		buffer[no]->start();

	return 1;	
}

int DSmultiAudio::stop(int no)
{
	if(buffer[no] != NULL)
		buffer[no]->stop();

	return 1;	
}