#include <MMSYSTEM.H>
#include <winbase.h>

#include "stdafx.h"
#include "BVtango.h"
#include "BVtangoDlg.h"
#include "Audio.h"
#include "TSaudioProc.h"

/////////////////////////////////
// module variables
/////////////////////////////////

////////////////////////////////////////////////////////////////
// common audio data
WAVEFORMATEX *format;	// format of data
MMRESULT err;			// audio error code
	
////////////////////////////////////////////////////////////////
// audio reading data

// handle for in port
HWAVEIN haudio_in = NULL;	

// reading Thread 
HANDLE hThread; 
DWORD dwThreadId;

// event for synchronize reading
HANDLE hEventIn;

// wave data header
WAVEHDR waveInHeader[2];	
int inHdrNo;

// reading state
int audioReadActive;
static int first = 1;

// audio
Audio *audio;

//FILE *log;

/////////////////////////////////////////////////////////////////
// audio writing data

AudioOutPort outport;

int audioInit(Audio *aud)
{
	char in_event_name[20] = "read";
	char out_event_name[20] = "write";
	int i;

	//log = fopen("audio.log", "w");
	
	audio = aud;
	
	format = (WAVEFORMATEX *)malloc(sizeof(WAVEFORMATEX));

	format->wFormatTag = WAVE_FORMAT_PCM;
	format->nChannels = 1;
	format->nSamplesPerSec = 8000;
    format->nAvgBytesPerSec = 16000; 
    format->nBlockAlign = 2; 
	format->wBitsPerSample = 16;
	format->cbSize = NULL;

	for(i=0; i<2; i++)
	{
		waveInHeader[i].lpData = new char[PORTION_SIZE];
		waveInHeader[i].dwBufferLength = PORTION_SIZE;
		waveInHeader[i].dwFlags = 0L;
		waveInHeader[i].dwLoops = 0L;
	}

	audioReadActive = 0;


	// create read event
	hEventIn = CreateEvent(NULL, TRUE, TRUE, in_event_name);

    // create reading thread
    hThread = CreateThread( 
        NULL,                        // no security attributes 
        0,                           // use default stack size  
        (LPTHREAD_START_ROUTINE) audioReadThread, // thread function 
        NULL,                // argument to thread function 
        0,                           // use default creation flags 
        &dwThreadId);                // returns the thread identifier 
	if (hThread == NULL) 
	{
		MessageBox(NULL, "CreateThread error", "Error", MB_OK);
		return 0;
	}


	outport.header[0].lpData = new char[PORTION_SIZE];
	outport.header[0].dwBufferLength = PORTION_SIZE;
	outport.header[1].lpData = new char[PORTION_SIZE];
	outport.header[1].dwBufferLength = PORTION_SIZE;

	// create write event
	outport.hEvent = CreateEvent(NULL, TRUE, TRUE, out_event_name);

	outport.active = 0;

	return 1;
}

void audioClose(void)
{
	int i;
	
	TerminateThread(hThread, 0);
	CloseHandle(hThread);

	free(format);
	CloseHandle(hEventIn);
	for(i=0; i<MAX_USERS; i++)
		CloseHandle(outport.hEvent);
	//fclose(log);
}

short audioGetLevel(char *buffer)
{
	int i;
	short max=0;

	for(i=0; i<PORTION_SAMPLES; i++)
	{
		if(((short *)buffer)[i] > max)
			max = ((short *)buffer)[i];
	}
	return max;
}

//////////////////////////////////////////////////////////////////
//
// Audio reading functions
//
//////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////
// audio reading init
int audioReadInit(void)
{
	//char text[100];
	char in_event_name[20] = "read";
	int i;

	first = 1;
	inHdrNo = 0;

	//SetThreadPriority( hThread,	THREAD_PRIORITY_TIME_CRITICAL);

	// opening audio in port
	err = waveInOpen(&haudio_in, WAVE_MAPPER, format, (DWORD)hEventIn, NULL, 
		CALLBACK_EVENT);
	if(err != MMSYSERR_NOERROR)
	{
		//wsprintf(text, "Can't open audio in port, error: %d", err);
		//MessageBox(NULL, text, "Error", MB_OK);
		return 0;
	}

	
	for(i=0; i<2; i++)
	{
		waveInHeader[i].dwFlags = 0L;
		waveInHeader[i].dwLoops = 0L;

		err = waveInPrepareHeader(haudio_in, &waveInHeader[i], sizeof(WAVEHDR));
		if(err != MMSYSERR_NOERROR)
		{
			//MessageBox(NULL, "Can't prepare audio header", "Error", MB_OK);
		}
	

		if(waveInAddBuffer(haudio_in, &waveInHeader[i], sizeof(WAVEHDR))
				!= MMSYSERR_NOERROR)
		{
			//MessageBox(NULL, "Can't add read audio buffer", "Error", MB_OK);
		}
	}

	audioReadActive = 1;

	ResetEvent(hEventIn);
	waveInStart(haudio_in);

	return 1;
}

///////////////////////////////////////////////////////////////////////////
// main audio reading thread
DWORD audioReadThread(LPDWORD lpdwParam)
{
	while(1)
	{
		WaitForSingleObject(hEventIn, INFINITE);

		if(waveInHeader[inHdrNo].dwFlags && WHDR_DONE)
		{
			audioReadChunk();
		}
		else
		{
			Sleep(10);
			//MessageBox(NULL, "Thread: audio read close", "Info", MB_OK);
		}
	}

	return 0;
}

///////////////////////////////////////////////////////////////////////////
// audio reading close function
void audioReadClose(void)
{
	//char text[100];
		

	audioReadActive = 0;

	waveInStop(haudio_in);

	err = waveInReset(haudio_in);
	if(err != MMSYSERR_NOERROR)
	{
		//wsprintf(text, "Can't reset audio in port %d", err);
		//MessageBox(NULL, text, "Error", MB_OK);
	}
	waveInUnprepareHeader(haudio_in, &waveInHeader[0], sizeof(WAVEHDR));
	waveInUnprepareHeader(haudio_in, &waveInHeader[1], sizeof(WAVEHDR));
	err = waveInClose(haudio_in);
	if(err != MMSYSERR_NOERROR)
	{
		//wsprintf(text, "Can't close audio input port, error: %d", err);
		//MessageBox(NULL, text, "Error", MB_OK);
	}
	// CloseHandle(haudio_in);
	haudio_in = NULL;

	//SetThreadPriority( hThread,	THREAD_PRIORITY_LOWEST);
	//ResetEvent(hEventIn);
}

///////////////////////////////////////////////////////////////////////////
// Reads audio portions from microphone and saves it to file
void audioReadChunk(void)
{
	void *buffer;

	waveInUnprepareHeader(haudio_in, &waveInHeader[inHdrNo], sizeof(WAVEHDR));
	
	buffer = malloc(PORTION_SIZE);
	memcpy(buffer, waveInHeader[inHdrNo].lpData, PORTION_SIZE);

	audioReadHandler(buffer);

	waveInHeader[inHdrNo].dwFlags = 0L;
	waveInHeader[inHdrNo].dwLoops = 0L;

	if(audioReadActive)
	{
		err = waveInPrepareHeader(haudio_in, &waveInHeader[inHdrNo], sizeof(WAVEHDR));
		if(err != MMSYSERR_NOERROR)
		{
			//MessageBox(NULL, "Can't prepare audio header", "Error", MB_OK);
		}
	
		ResetEvent(hEventIn);

		if(waveInAddBuffer(haudio_in, &waveInHeader[inHdrNo], sizeof(WAVEHDR))
				!= MMSYSERR_NOERROR)
		{
			//MessageBox(NULL, "Can't add read audio buffer", "Error", MB_OK);
		}

		inHdrNo = (inHdrNo + 1) % 2;
	}
	
}

///////////////////////////////////////////////////////////////////////////
// Handler for read audio chunk
void audioReadHandler(void *buffer)
{
	if(audioReadActive)
	{
		// TODO: place here your read audio handler 
		audio->SendData((char *)buffer);
	}
	
	free(buffer);
}
 
///////////////////////////////////////////////////////////////////////////
// starting handling of read audio buffers
void audioReadStart(void)
{
	audioReadActive = 1;
}

///////////////////////////////////////////////////////////////////////////
// stop handling of read audio buffers
void audioReadStop(void)
{
	audioReadActive = 0;
}


//////////////////////////////////////////////////////////////////
//
// Audio writing functions
//
//////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////
// audio writing init
int audioWriteInit(void)
{
	outport.first = 1;
	outport.handle = NULL;
	outport.headerNo = 0;
	outport.active = 1;

	if(outport.handle == NULL)
	{
		err = waveOutOpen(&outport.handle, WAVE_MAPPER, format, (DWORD)outport.hEvent, NULL, 
			CALLBACK_EVENT);
		if(err != MMSYSERR_NOERROR)
		{
			//wsprintf(text, "Can't open audio out port %d", err);
			//MessageBox(NULL, text, "Error", MB_OK);
			return 0;
		}	

	}

	return 1;
}

///////////////////////////////////////////////////////////////////////////
// audio writnig close
void audioWriteClose(void)
{
	char text[100];
	
	err = waveOutReset(outport.handle);
	if(err != MMSYSERR_NOERROR)
	{
		wsprintf(text, "Can't reset audio out port %d", err);
		MessageBox(NULL, text, "Error", MB_OK);
	}
	waveOutUnprepareHeader(outport.handle, &outport.header[0], sizeof(WAVEHDR));
	waveOutUnprepareHeader(outport.handle, &outport.header[1], sizeof(WAVEHDR));
	err = waveOutClose(outport.handle);
	if(err != MMSYSERR_NOERROR)
	{
		wsprintf(text, "Can't close audio out port %d", err);
		MessageBox(NULL, text, "Error", MB_OK);
	}
	outport.handle = NULL;
}


///////////////////////////////////////////////////////////////////////////
// setting audio volume
void audioWriteVolume(UINT vol)
{
	waveOutSetVolume(outport.handle, vol);
}

///////////////////////////////////////////////////////////////////////////
// Puts audio portion to speaker
void audioWriteChunk(void *buffer)
{
	static int gap = 0;
	int nowait = 0;
	
	if(!outport.first)
	{
		gap++;
		if(gap == 10000)
		{
			waveOutReset(outport.handle);
			gap = 0;
		}

		ResetEvent(outport.hEvent);

		if(waveOutWrite(outport.handle, &(outport.header[outport.headerNo]), sizeof(WAVEHDR))
			!= MMSYSERR_NOERROR)
			MessageBox(NULL, "Can't write audio", "Error", MB_OK);
		
		outport.headerNo = (outport.headerNo + 1) % 2;

		if(!(outport.header[outport.headerNo].dwFlags & WHDR_DONE))
			WaitForSingleObject(outport.hEvent, INFINITE);
		waveOutUnprepareHeader(outport.handle, &(outport.header[outport.headerNo]), sizeof(WAVEHDR));
	}
	else
		outport.first = 0;

	outport.header[outport.headerNo].dwFlags = 0L;
	outport.header[outport.headerNo].dwLoops = 0L;

	memcpy(outport.header[outport.headerNo].lpData, buffer, PORTION_SIZE); 

	err = waveOutPrepareHeader(outport.handle, &(outport.header[outport.headerNo]), sizeof(WAVEHDR));
	if(err != MMSYSERR_NOERROR)
	{
		//MessageBox(NULL, "Can't prepare audio out header", "Error", MB_OK);
	}
}


