/*********************************************************************
 *
 * File:         session_manage.c
 *
 * Description:  Session management module for conference engine
 *
 * Author:       Tomasz Stachowiak, NPAC
 *
 * Date:         10-15-96
 *
 *********************************************************************/
#include "stdafx.h"

#include <stdio.h>

#include "global.h"
#include "session_manage.h"

extern FILE *log;
extern char localHost[];   /* local host name */
Session session;

void nic(...)
{
}
/************************************************************************
 *
 * Additional procedures
 *
 ************************************************************************/
int addr2name(struct sockaddr_in *addr, char *name)
{
  struct hostent *hostinfo;
  char *tmp;
  struct in_addr address;

  hostinfo = gethostbyaddr((const char *)&addr->sin_addr, sizeof(struct in_addr), AF_INET);
  if(hostinfo == NULL)
    {
	  address.s_addr = addr->sin_addr.s_addr;
	  tmp = inet_ntoa(address);
	  if(tmp == NULL)
		return 0;
	  strcpy(name, tmp);
	  return 1;
    }
  strcpy(name, hostinfo->h_name);
  return 1;
}


int start_process(char *path, PROCESS_INFORMATION *pinfo)
{
	STARTUPINFO sinfo;

	sinfo.cb = sizeof(STARTUPINFO);
	sinfo.lpReserved = NULL;
	sinfo.lpDesktop = NULL;
	sinfo.lpTitle = NULL;
	sinfo.dwFlags = STARTF_FORCEONFEEDBACK;
	sinfo.cbReserved2 = 0;
	sinfo.lpReserved2 = NULL;

	if(CreateProcess(path, NULL, NULL, NULL, FALSE, 
					NORMAL_PRIORITY_CLASS | DETACHED_PROCESS, NULL, NULL, 
					&sinfo, pinfo)
		== FALSE)
			PRINTF(log, "Create process error");
	
	printf("Process %s created\n", path);

	return 1;
}

int newMember(Participant *member)
{
  int pos;

  pos = member->handle;
  session.numMembers++;
  session.memberActive[pos] = 1;
  session.member[pos].handle = pos;
  strcpy(session.member[pos].user, member->user);
  session.member[pos].host = member->host;

  return 1;
}

/************************************************************************
 *
 * Initialization and creation
 *
 ************************************************************************/
int sessionInit()
{
  int i;

  session.status = statusInit;
  session.lastMember = -1;
  session.numMembers = 0;
  session.localMember = -1;
  for(i=0; i<MAX_MEMBERS; i++)
    session.memberActive[i] = 0;

  return 1;
}

void setSessionID(int id)
{
  session.id = id;
}

int sessionCreate(Participant *initiator)
{
  if(session.status == statusInit)
    {
      PRINTF(log, "Engine: session initiator %s@%d\n", initiator->user, initiator->host);
      session.status = statusCreated;

      newMember(initiator);
      return 1;
    }
  else
    {
      PRINTF(log, "Engine: session already created -- status %d\n", session.status);
      return 0;
    }
}

void sessionAbandon()
{
  sessionInit();
  session.status = statusAbandoned;
}

/************************************************************************
 *
 * Members management
 *
 ************************************************************************/
int sessionAddMember(Participant *newmember)
{
  if(session.status == statusCreated)
    {
      if(session.lastMember < MAX_MEMBERS-1)
	{
	  if(getUserHandle(newmember) == -1)
	    {
	      PRINTF(log, "Engine: session member %s@%d\n", 
		     newmember->user, newmember->host);
	      newMember(newmember);
	      return 1;
	    }
	  else 
	    return 0;
	}
    }
  return 0;
}

void sessionDeleteMember(int no)
{
  session.memberActive[no] = 0;
  session.numMembers--;
}

void sessionDeletePart(Participant *part)
{
	int i;

	for (i=0; i < MAX_MEMBERS; i++)
	{
		if((session.memberActive[i] == 1) && (part->host == session.member[i].host))
			sessionDeleteMember(i);
	}
}

void sessionSetLocalMember(int no)
{
  PRINTF(log, "Engine: session set local member %d\n", no);
  session.localMember = no;
  //sendAppMessage(-1, indLocalSessionLocalMember, 
//	     (char *)&session.localMember, sizeof(int));
}


/************************************************************************
 *
 * Session query
 *
 ************************************************************************/
/**********************************************************************
 * Members queries 
 */
int sessionGetNoMembers()
{
  return session.numMembers;
}


void sessionGetMembers(Participant *parts)
{
  int i, no = 0;

  for(i=0; i<MAX_MEMBERS; i++)
    {
      if(session.memberActive[i])
	{
	  memcpy(parts, &session.member[i], sizeof(Participant));
	  //parts->host = htonl(parts->host);
	  parts++;
	}
    }
}

int getReceiverData(int userNo, MsgHeader *header)
{
  if((session.memberActive[userNo]) && (userNo != session.localMember))
    {
      memcpy(&header->receiver, &session.member[userNo], sizeof(Participant));
      return 1;
    }
  else
    return 0;
}


int getUserData(int userNo, struct sockaddr_in *addr, Participant *part)
{
  if((session.memberActive[userNo]) && (userNo != session.localMember))
    {
      memcpy(addr, &session.memberAddr[userNo], sizeof(struct sockaddr_in));
      memcpy(part, &session.member[userNo], sizeof(Participant));
      return 1;
    }
  else
    return 0;
}

int getUserHandle(Participant *part)
{
  int i;

  for(i=0; i<MAX_MEMBERS; i++)
    {
      if(session.memberActive[i])
	if(   (session.member[i].host == part->host)
	   && (strcmp(session.member[i].user, part->user) == 0))
	      return i;
    }

  return -1;
}

int getLocalHandle()
{
  return session.localMember;
}

/**********************************************************************
 * Status queries 
 */
int sessionCreated()
{
  if(session.status == statusCreated)
    return 1;
  else
    return 0;
}

int sessionInitialized()
{
  if(session.status == statusInit)
    return 1;
  else
    return 0;
}

int sessionAbandoned()
{
  if(session.status == statusAbandoned)
    return 1;
  else
    return 0;
}

int getSessionID()
{
  return session.id;
}

