/*
 * File         : BV_AppReadThread.C
 * Author       : Tomasz Stachowiak
 * Created      : Fri Jul 24 16:33:30 1998
 * Revision     : $Revision$
 * Updated      : $Date$
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

#include <stdio.h>

#include "stdafx.h"
#include "BV_AppReadThread.h"
#include "BV_Application.h"
#include "rtp.h"

BV_AppReadThread::BV_AppReadThread(void *app)
{
  app_ = app;
}

int BV_AppReadThread::run()
{
	struct timeval timeout;
	fd_set set;
	RTPheader header;
	char *msg;
	int size;
	int sockfd;
	char *data;
	struct sockaddr_in addr;
	int len;

	sockfd = ((BV_Application *)app_)->get_data_sock();
	len = sizeof(struct sockaddr_in);
	while(running)
    {
		timeout.tv_sec = 0;
		timeout.tv_usec = 100;
		FD_ZERO(&set);
		FD_SET(sockfd, &set);

		if(select(sockfd + 1, &set, 0, 0, &timeout) > 0)
		{
			if(FD_ISSET(sockfd, &set))
			{
				data = new char[MAX_UDP_SIZE];
				size = recvfrom(sockfd, data, MAX_UDP_SIZE, 0, (struct sockaddr *)&addr, &len);
				if(size > 0)
				{
					size = size - RTP_HEADER_SIZE;
					readRTPheader(&header, data);
					msg = data + RTP_HEADER_SIZE;
					/*printf("Get data type %d from user %d\n", header->pt, header->ssrc);*/
					((BV_Application *)app_)->process_message(&header, msg, size);
				}
				else
				{
					int err = WSAGetLastError();
					return -1;
				}
	      
				delete data;
			}
		}
    }

	return 0;
}

void BV_AppReadThread::readRTPheader(RTPheader *hdr, char *data)
{
	hdr->type = (data[0] & 192) >> 6;
	hdr->p = (data[0] & 32) >> 5;
	hdr->x = (data[0] & 16) >> 4;
	hdr->cc = data[0] & 15;
	hdr->m = (data[1] & 128) >> 7;
	hdr->pt = data[1] & 127;
	hdr->seq = ntohs(*((short *)&data[2]));
	hdr->ts = ntohl(*((int *)&data[4]));
	hdr->ssrc = ntohl(*((int *)&data[8]));
}

