/*
 * File         : BV_Application.C
 * Author       : Tomasz Stachowiak
 * Created      : Fri Jul 24 17:19:28 1998
 * Revision     : $Revision$
 * Updated      : $Date$
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

#include <stdio.h>

#include "stdafx.h"
#include "BV_Member.h"
#include "BV_Session.h"
#include "BV_AppReadThread.h"
#include "BV_Application.h"

BV_Application::BV_Application(int port)
{
  session_ = BV_Session::instance();
  data_sock_ = -1;
  port_ = port;
  read_thread_ = new BV_AppReadThread(this);
  sequence = 0;
}

BV_Application::~BV_Application()
{
  delete read_thread_;
  session_->destroy();
}

int BV_Application::init()
{
  struct sockaddr_in addr;

  if ((data_sock_ = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
    {
      return -1;
    }
  
  ZeroMemory(&addr, sizeof(struct sockaddr_in));
  addr.sin_family      = AF_INET;
  addr.sin_addr.s_addr = htonl(INADDR_ANY);
  addr.sin_port        = htons(port_);
  
  if(bind(data_sock_, (struct sockaddr *)&addr, sizeof(struct sockaddr_in)) < 0)
    {
      return -1;
    }
  
  if(read_thread_->start() < 0)
    return -1;

  return 0;
}

void BV_Application::close()
{
  read_thread_->stop();
  closesocket(data_sock_);
}

int BV_Application::send_data(int type, char *data, int len)
{
	char *ncsdata;
	char *tmp;
	int size;
	int sent = 0;
	int i;
	RTPheader header;

	size = RTP_HEADER_SIZE + len;
	ncsdata = new char[size];
	fill_rtp_header(&header, type);
	tmp = ncsdata + RTP_HEADER_SIZE;
	memcpy(tmp, data, len); 
	header.ssrc = session_->local_id_;
	rtp_header_to_bytes(&header, ncsdata);

	for(i = 0; i < MAX_MEMBERS; i++)
	{
		if((session_->members[i] != NULL) && (session_->members[i]->id_ != session_->local_id_))
		{
			if(send_packet(data_sock_, session_->members[i]->host_, port_, ncsdata, size) < 0)
				return -1;
		}
		i++;
    }

  delete ncsdata;

  return 0;
}

int BV_Application::send_packet(int sock, int hostid, int port, char *msg, int len)
{
  struct sockaddr_in addr;

  ZeroMemory(&addr, sizeof(struct sockaddr_in));
  addr.sin_family      = AF_INET;
  addr.sin_addr.s_addr = hostid;
  addr.sin_port        = htons(port);

  if(sendto(sock, msg, len, 
         0, (struct sockaddr *)&addr, sizeof(struct sockaddr_in)) < 0)
  {
		int err = WSAGetLastError();
		return -1;  
  }

  return 0;
}

void BV_Application::fill_rtp_header(RTPheader *header, unsigned int payload)
{
  header->type = 2;
  header->p = 0;
  header->x = 0;
  header->cc = 0;
  header->m = 0;
  header->pt = payload;
  header->seq = sequence++;
  header->ts = get_time_stamp();
  header->ssrc = 0;  
}

unsigned int BV_Application::get_time_stamp()
{  
  return GetTickCount();
}

int BV_Application::get_data_sock()
{
  return data_sock_;
}

void BV_Application::rtp_header_to_bytes(RTPheader *hdr, char *data)
{
	short tmp1;
	int tmp2;
	int i;

	for(i=0; i<RTP_HEADER_SIZE; i++)
		data[i] = 0;
	
	data[0] = data[0] | ((hdr->type << 6) & 192);
	data[0] = data[0] | ((hdr->p << 5) & 32);
	data[0] = data[0] | ((hdr->x << 4) & 16);
	data[0] = data[0] | (hdr->cc & 15);
	data[1] = data[1] | ((hdr->m << 7) & 128);
	data[1] = data[1] | (hdr->pt & 127);
	tmp1 = htons(hdr->seq);
	memcpy(&data[2], &tmp1, 2);
	tmp2 = htonl(hdr->ts);
	memcpy(&data[4], &tmp2, 4);
	tmp2 = htonl(hdr->ssrc);
	memcpy(&data[8], &tmp2, 4);
}

void BV_Application::set_read_priority(int priority)
{
	read_thread_->set_priority(priority);
}