#include "stdafx.h"
#include "h261.h"
#include "h263.h"
#include "convert.h"
#include "video.h"
#include "options.h"
#include "yuvrgb24.h"
#include "UserData.h"
#include "rtp.h"
#include "DisplayDlg.h"
#include "BV_Member.h"

BV_Member::BV_Member()
{
	id_ = -1;
	host_ = -1;
	dpy_ = NULL;
	astat_ = NULL;
	vstat_ = NULL;
	audioOn_ = FALSE;
	videoOn_ = FALSE;
	acompr_ = -1;
	vcompr_ = -1;
	firstFrame = 1;
	strcpy(string_, "Unknown");
}

BV_Member::BV_Member(int id, int host, char *name)
{
	id_ = id;
	host_ = host;
	strcpy(name_, name);
	dpy_ = NULL;
	vstat_ = NULL;
	astat_ = NULL;
	audioOn_ = FALSE;
	videoOn_ = FALSE;
	acompr_ = -1;
	vcompr_ = -1;
	firstFrame = 1;
	getHostName(host_, string_);
}

BV_Member::~BV_Member()
{
	closeDisplay();
	if(astat_ != NULL)
		delete astat_;
	if(vstat_ != NULL)
		delete vstat_;

}

void BV_Member::setVideo(void *video)
{
	video_ = video;
}


CDialog *BV_Member::openDisplay()
{
	if(dpy_ == NULL)
	{
		dpy_ = new CDisplayDlg();

		if(!dpy_->Create(IDD_DISPLAY))
			MessageBox(NULL, "Error creating window", "Error", MB_OK);

		dpy_->open(this);

		videoOn_ = TRUE;
	}

	return dpy_;
}


void BV_Member::closeDisplay()
{
	if(dpy_ != NULL)
	{
		dpy_->close();
		dpy_->DestroyWindow();
		delete dpy_;
		dpy_ = NULL;
	}
}

void BV_Member::showDisplay()
{
	if(dpy_ != NULL)
		dpy_->ShowWindow(SW_SHOW);
}

void BV_Member::hideDisplay()
{
	if(dpy_ != NULL)
		dpy_->ShowWindow(SW_HIDE);
}


int BV_Member::displayFrame(int compr, unsigned char *data, int len)
{
	unsigned char *yuv;
	char *image;
	unsigned char **pYUV;

	if(dpy_ != NULL)
	{
		image = (char *)GlobalAlloc(GPTR, QCIF_LUM_SIZE*3);

		switch(compr)
		{
		case YUV:
			ConvertYUVtoRGB(data, &data[QCIF_LUM_SIZE], &data[QCIF_LUM_SIZE + QCIF_CR_SIZE],
							(unsigned char *)image, 176, 144);
			break;

		case YUV9:
			yuv = (unsigned char *)GlobalAlloc(GPTR, QCIF_LUM_SIZE*3/2);
			YVU9toYUV411(data, yuv, &yuv[QCIF_LUM_SIZE], &yuv[QCIF_LUM_SIZE + QCIF_CR_SIZE], 176, 144);
			ConvertYUVtoRGB(yuv, &yuv[QCIF_LUM_SIZE], &yuv[QCIF_LUM_SIZE + QCIF_CR_SIZE],
							(unsigned char *)image, 176, 144);
			GlobalFree(yuv);
			break;

		case H261:
			yuv = (unsigned char *)GlobalAlloc(GPTR, QCIF_LUM_SIZE*3/2);
			WaitForSingleObject(((Video *)video_)->hCodecEvent, INFINITE);
			ResetEvent(((Video *)video_)->hCodecEvent);
			if(firstFrame)
			{
				p64DecodeFrame((char *)data, len, (char *)yuv, 1, id_);
				firstFrame = 0;
			}
			else
				p64DecodeFrame((char *)data, len, (char *)yuv, 0, id_);
			SetEvent(((Video *)video_)->hCodecEvent);
			ConvertYUVtoRGB(yuv, &yuv[QCIF_LUM_SIZE], &yuv[QCIF_LUM_SIZE + QCIF_CR_SIZE],
							(unsigned char *)image, 176, 144);
			GlobalFree(yuv);
			break;

		case H263:
			pYUV = H263decodeFrame((char *)data, len, id_);
			ConvertYUVtoRGB(pYUV[0], pYUV[1], pYUV[2],
						(unsigned char *)image, 176, 144);
			break;
		}

		dpy_->display(image);
		GlobalFree(image);

		updateDpyStat();
	}

	return 0;
}

void BV_Member::updateDpyStat()
{
	static int count = 0;

	if(count == 0)
	{
		if(vstat_ != NULL)
		{
			dpy_->setKbps(vstat_->getKbps());
			dpy_->setFps(vstat_->getFps());
		}
		if(audioOn_)
			dpy_->SetStatus("Audio On");
		else
			dpy_->SetStatus("Audio Off");
	}

	count = (count + 1) % 5;
}

void BV_Member::videoStart()
{
	videoOn_ = TRUE;
}

void BV_Member::videoStop()
{
	videoOn_ = FALSE;
}

void BV_Member::videoFrame(RTPheader *header, int len)
{
	videoOn_ = TRUE;
	if(vstat_ == NULL)
		vstat_ = new UserData(header, len);
	else
		vstat_->frame(header, len);
}
		
void BV_Member::audioFrame(RTPheader *header, int len)
{
	audioOn_ = TRUE;
	if(astat_ == NULL)
		astat_ = new UserData(header, len);
	else
		astat_->frame(header, len);
}

void BV_Member::audioEnd()
{
	audioOn_ = FALSE;
}

void BV_Member::updateStat()
{
}

void BV_Member::displayClosed()
{
}

char *BV_Member::toString()
{
	return string_;
}

char *BV_Member::toSimpleString()
{
	return string_;
}

