//////////////////////////////////////////////////////////////////////////
//
// File					: convert.cpp
//
// Description			: Convertion functions between RGB 16 and YUV 411
//
///////////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include <stdio.h>
#include "convert.h"

#define COEFFR 1.1
#define COEFFG 1.1
#define COEFFB 1.1

int mirror = 1;

unsigned char getB(WORD rgb)
{
	unsigned char result;

	result = ((unsigned char)(rgb & 0x1F)) << 3;
	return (float)COEFFB*(float)result;
}

unsigned char getG(WORD rgb)
{
	unsigned char result;

	result = ((unsigned char)((rgb >> 5) & 0x1F)) << 3;
	return (float)COEFFG*(float)result;
}

unsigned char getR(WORD rgb)
{
	unsigned char result;

	result = ((unsigned char)((rgb >> 10) & 0x1F)) << 3;
	return (float)COEFFR*(float)result;
}

void putB(WORD *rgb, char r)
{
	WORD pom;

	pom = (r>>3) & 0x001F;
	*rgb = *rgb | pom;
}

void putG(WORD *rgb, char r)
{
	WORD pom;

	pom = (r << 2) & 0x03E0;
	*rgb = *rgb | pom;
}

void putR(WORD *rgb, char r)
{
	WORD pom;

	pom = (r << 7) & 0xEC00;
	*rgb = *rgb | pom;
}



void RGB16toYUV(WORD *input, 
			 unsigned char *y, unsigned char *u, unsigned char *v, 
			 int width, int height)
{
	int i, j;
	short cru[4], crv[4], lum;
	int QCIFwidth;

	QCIFwidth = 176;
	for(i=0; i<height; i++)
	{
		for(j=0; j<width; j++)
		{
			lum =	.299*(unsigned float)getR(input[i*width + j]) 
				+	.587*(unsigned float)getG(input[i*width + j])
				+	.114*(unsigned float)getB(input[i*width + j]);
			
			lum = (float)((float)219/(float)255)*(unsigned float)lum + 16;
			y[(height-i)*QCIFwidth + (j)] = lum;

			if(((i%2) == 1) && ((j%2) == 1))
			{
				cru[0] =	(-0.1687)*(unsigned float)getR(input[i*width + j])
					+		(-0.3313)*(unsigned float)getG(input[i*width + j])
					+		(0.5)	 *(unsigned float)getB(input[i*width + j]);
				
				cru[1] =	(-0.1687)*(unsigned float)getR(input[i*width + j - 1])
					+		(-0.3313)*(unsigned float)getG(input[i*width + j - 1])
					+		(0.5)	 *(unsigned float)getB(input[i*width + j - 1]);
				
				cru[2] =	(-0.1687)*(unsigned float)getR(input[i*(width-1) + j])
					+		(-0.3313)*(unsigned float)getG(input[i*(width-1) + j])
					+		(0.5)	 *(unsigned float)getB(input[i*(width-1) + j]);
				
				cru[3] =	(-0.1687)*(unsigned float)getR(input[i*(width-1) + j - 1])
					+		(-0.3313)*(unsigned float)getG(input[i*(width-1) + j - 1])
					+		(0.5)	 *(unsigned float)getB(input[i*(width-1) + j - 1]);

				u[(((height-i)/2)*(QCIFwidth/2)) + ((j)/2)] = (unsigned char)((((float)224/(float)255)*(unsigned float)((cru[0] + cru[1] + cru[2] + cru[3])/4)) + 128);

				crv[0] =	(0.5)    *(unsigned float)getR(input[i*width + j])
					+		(-0.4187)*(unsigned float)getG(input[i*width + j])
					+		(-0.0813)*(unsigned float)getB(input[i*width + j]);
				
				crv[1] =	(0.5)    *(unsigned float)getR(input[i*width + j - 1])
					+		(-0.4187)*(unsigned float)getG(input[i*width + j - 1])
					+		(-0.0813)*(unsigned float)getB(input[i*width + j - 1]);
				
				crv[2] =	(0.5)    *(unsigned float)getR(input[i*(width-1) + j])
					+		(-0.4187)*(unsigned float)getG(input[i*(width-1) + j])
					+		(-0.0813)*(unsigned float)getB(input[i*(width-1) + j]);
				
				crv[3] =	(0.5)    *(unsigned float)getR(input[i*(width-1) + j - 1])
					+		(-0.4187)*(unsigned float)getG(input[i*(width-1) + j - 1])
					+		(-0.0813)*(unsigned float)getB(input[i*(width-1) + j - 1]);

				v[(((height-i)/2)*(QCIFwidth/2)) + ((j)/2)] = (unsigned char)((((float)224/(float)255)*(unsigned float)((crv[0] + crv[1] + crv[2] + crv[3])/4)) + 128);
			}
		}
	}
}


void setMirror()
{
	mirror = 1;
}

void resetMirror()
{
	mirror = 0;
}

void RGB24toYUV(unsigned char *input, 
			 unsigned char *y, unsigned char *u, unsigned char *v, 
			 int width, int height)
{
	register int i, j;
	register unsigned char *in;
	register short cru, crv, lum;
	register unsigned char *y1 = y, *u1 = u, *v1 = v; 
	const int QCIFwidth = 176;
	const int QCIFheigth = 144;
	static BOOL  first = TRUE;
    static float  mult299[1024], mult587[1024], mult114[1024];
    static float  mult16874[1024], mult33126[1024], mult5[1024];
    static float mult41869[1024], mult08131[1024];

    //if(mirror)
	//	in = input + width*height*3;
	//else
		in = input + width*(height - 1)*3;

	if ( first ) 
	{
        register int index;
		register int maxValue;

		maxValue = 256;

        for ( index = 0; index <= maxValue; index++ ) 
		{
			mult299[index] = index*0.29900;
			mult587[index] = index*0.58700;
			mult114[index] = index*0.11400;
			mult16874[index] = -0.16874*index;
			mult33126[index] = -0.33126*index;
			mult5[index] = index*0.50000;
			mult41869[index] = -0.41869*index;
			mult08131[index] = -0.08131*index;
		}
	
		first = FALSE;
    }

	for(i=0; i<QCIFheigth; i++)
	{
		for(j=0; j<QCIFwidth; j++)
		{
			lum =	 mult114[*(in)] 
				+	 mult587[*(in + 1)]
				+	 mult299[*(in + 2)];
			
			*(y1++) = (219*lum/255) + 16;

			if(((i%2) == 1) && ((j%2) == 1))
			{
				cru =	 mult5[*(in)] 
					+	 mult33126[*(in + 1)]
					+	 mult16874[*(in + 2)];

				*(u1++) = (224*cru/255) + 128;

				crv =	 mult08131[*(in)] 
					+	 mult41869[*(in + 1)]
					+	 mult5[*(in + 2)];

				*(v1++) = (224*crv/255) + 128;

			}

	//		if(mirror)
	//			in -= 3;
	//		else
				in += 3;
		}

	//	if(mirror)
	//		in -= 3*(width - QCIFwidth);
	//	else
			in -= 3*(width + QCIFwidth);

	}
}

void RGB24toQCIF(char *input, int width, int height, char *output)
{
	char *tmpout = output;
	char *tmpin = input;
	int QCIFwidth, QCIFheigth;
	int i;

	QCIFwidth = 176;
	QCIFheigth = 144;
	for(i=0; i<QCIFheigth; i++)
	{
		memcpy(tmpout, tmpin, 3*QCIFwidth);
		tmpout += 3*QCIFwidth;
		tmpin += 3*width;
	}
}

void YVU9toYUV411(unsigned char *input, 
			 unsigned char *y, unsigned char *u, unsigned char *v, 
			 int width, int heigth)
{
	int i, j;
	int QCIFwidth, QCIFheigth;
	unsigned char *cr1, *cr2;
	unsigned char pom;
	
	cr1 = &input[width * heigth];
	cr2 = &input[width * heigth*17/16];

	QCIFwidth = 176;
	QCIFheigth = 144;

	for(i=0; i<QCIFheigth; i++)
	{
		for(j=0; j<QCIFwidth; j++)
		{
			y[i*QCIFwidth + j] = input[i*width + j];
		}
	}
	
	/*
	for(i=0; i<QCIFheigth/2; i++)
	{
		for(j=0; j<QCIFwidth/2; j++)
		{
				u[i*QCIFwidth/2 + j] = cr2[(i*width + j)/8];
				v[i*QCIFwidth/2 + j] = cr1[(i*width + j)/8];
		}
	}
	*/

	for(i=0; i<QCIFheigth/4; i++)
		for(j=0; j<QCIFwidth/4; j++)
		{
			pom = cr2[i*width/4 + j];
			u[i*QCIFwidth + j*2] = pom;
			u[i*QCIFwidth + j*2 + 1] = pom;
			u[(2*i+1)*(QCIFwidth/2) + j*2] = pom;
			u[(2*i+1)*(QCIFwidth/2) + j*2 + 1] = pom;
	
			pom = cr1[i*width/4 + j];
			v[i*QCIFwidth + j*2] = pom;
			v[i*QCIFwidth + j*2 + 1] = pom;
			v[(2*i+1)*(QCIFwidth/2) + j*2] = pom;
			v[(2*i+1)*(QCIFwidth/2) + j*2 + 1] = pom;
		}
  }


void resizeYUV9(char *image, int x0, int y0, char *out, int x1, int y1)
{
	char *lum1, *lum0, *cr1, *cr0, *cb1, *cb0;
	int i;

	lum1 = out;
	lum0 = image;
	cr0 = &image[x0*y0];
	cr1 = &out[x1*y1];
	cb0 = &image[x0*y0*17/16];
	cb1 = &out[x1*y1*17/16];
	
	for(i=0; i<y1; i++)
	{
		memcpy(lum1, lum0, x1);
		lum1 += x1;
		lum0 += x0;
	}
	for(i=0; i<y1/4; i++)
	{
		memcpy(cr1, cr0, x1/4);
		cr0 += x0/4;
		cr1 += x1/4;

		memcpy(cb1, cb0, x1/4);
		cb0 += x0/4;
		cb1 += x1/4;
	}
}

