// DisplayDlg.cpp : implementation file
//

#include <vfw.h>
#include <wingdi.h>

#include "stdafx.h"
#include "BuenaVista.h"
#include "yuvrgb24.h"
#include "BV_Member.h"
#include "DisplayDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

BITMAPINFO *CDisplayDlg::bmpInfo = NULL;
int CDisplayDlg::mode = COL256;

/////////////////////////////////////////////////////////////////////////////
// CDisplayDlg dialog


CDisplayDlg::CDisplayDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CDisplayDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CDisplayDlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}


void CDisplayDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CDisplayDlg)
	DDX_Control(pDX, IDC_STATUS, m_status);
	DDX_Control(pDX, IDC_KBPS, m_kbps);
	DDX_Control(pDX, IDC_FPS, m_fps);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CDisplayDlg, CDialog)
	//{{AFX_MSG_MAP(CDisplayDlg)
	ON_WM_SIZE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDisplayDlg message handlers

void CDisplayDlg::OnSize(UINT nType, int cx, int cy) 
{
	CDialog::OnSize(nType, cx, cy);

	if(m_status.m_hWnd != 0)
	{
		m_status.MoveWindow(0, cy-40, cx, 20);
		m_fps.MoveWindow(0, cy-20, cx/2, 20);
		m_kbps.MoveWindow(cx/2, cy-20, cx/2, 20);
	}
}

BOOL CDisplayDlg::OnInitDialog() 
{
	RECT rect;

	CDialog::OnInitDialog();
	
	this->GetClientRect(&rect);
	if(m_status.m_hWnd != 0)
	{
		m_status.MoveWindow(0, rect.bottom-40, rect.right, 20);
		m_fps.MoveWindow(0, rect.bottom-20, rect.right/2, 20);
		m_kbps.MoveWindow(rect.right/2, rect.bottom-20, rect.right/2, 20);
	}
	
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CDisplayDlg::SetStatus(char *text)
{
	m_status.SetSel(0, -1);
	m_status.ReplaceSel(text);

}

void CDisplayDlg::setFps(double fps)
{
	char text[30];

	sprintf(text, "%4.1f fps", fps);
	m_fps.SetSel(0, -1);
	m_fps.ReplaceSel(text);

}

void CDisplayDlg::setKbps(double kbps)
{
	char text[30];

	sprintf(text, "%4.0f kbps", kbps);
	m_kbps.SetSel(0, -1);
	m_kbps.ReplaceSel(text);

}

void CDisplayDlg::globalInit()
{
	DEVMODE devm;

	bmpInfo = (BITMAPINFO *)GlobalAlloc(GPTR, sizeof(BITMAPINFO));

	// BITMAPINFO structure initialization (video format)
	bmpInfo->bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
	bmpInfo->bmiHeader.biWidth = 176; //176;
	bmpInfo->bmiHeader.biHeight = 144; //144;
	bmpInfo->bmiHeader.biPlanes = 1;
	bmpInfo->bmiHeader.biBitCount = 24;
	bmpInfo->bmiHeader.biCompression = BI_RGB;
	bmpInfo->bmiHeader.biSizeImage = 0;
	bmpInfo->bmiHeader.biXPelsPerMeter = 0;
	bmpInfo->bmiHeader.biYPelsPerMeter = 0;	
	bmpInfo->bmiHeader.biClrUsed = 0;
	bmpInfo->bmiHeader.biClrImportant = 0;	

	init_dither_tab();

	devm.dmBitsPerPel = GetDeviceCaps(CreateDC("DISPLAY", NULL, NULL, NULL), BITSPIXEL);
	if(devm.dmBitsPerPel <= 8)
		mode = COL256;
	else
		mode = TRUECOL;
}

int CDisplayDlg::open(void *member)
{
	member_ = member;

	SetWindowPos(&wndTopMost, 40 * ((BV_Member *)member_)->id_ + 100, 100, 176, 164, SWP_NOSIZE);
	SetWindowText(((BV_Member *)member_)->toSimpleString());

	if(mode == COL256)
		ModifyStyle(WS_SIZEBOX, 0);

	cDc = new CClientDC(this);
	cDc->SetMapMode(MM_TEXT);
	hDc = cDc->GetSafeHdc();

	SetStretchBltMode(hDc, HALFTONE);	
	UnrealizeObject(GetCurrentObject(hDc, OBJ_BRUSH));
	SetBrushOrgEx(hDc, 0, 0, NULL);
	SelectObject(hDc, GetCurrentObject(hDc, OBJ_BRUSH));

	if(mode == COL256)
	{
		hDraw = DrawDibOpen();

		if(!DrawDibBegin(hDraw, hDc, 176, 144, &bmpInfo->bmiHeader, -1, -1, DDF_HALFTONE ))
			::MessageBox(NULL, "Draw Dib begin error", "Error", MB_OK);
	}

	return 0;
}

void CDisplayDlg::close()
{
	delete cDc;
	if(mode == COL256)
		DrawDibClose(hDraw);
}

int CDisplayDlg::display(char *image)
{
	RECT rect;
	int err;

	GetClientRect(&rect);

	if(mode == TRUECOL)
	{
		err = StretchDIBits(hDc, 0, 0, rect.right, rect.bottom-40, 0, 0, 176, 144,
				            image, bmpInfo, DIB_RGB_COLORS, SRCCOPY);

		if(err == GDI_ERROR)
		{
			::MessageBox(NULL, "GDI error", "Error", MB_OK);
			return -1;
		}
	}

	if(mode == COL256)
	{
		DrawDibDraw(hDraw, hDc, 0, 0, 176, 144, &bmpInfo->bmiHeader, image, 
				    0, 0, 176, 144, DDF_HALFTONE);
	}

	return 0;
}


